function entity:worldSize() 
    return self:defaultSize() * vec2(self.scale.x, self.scale.y)
end

function entity:defaultSize()
    if self.sprite and self.scene then
        return (vec2(select(3, self.sprite:rect()), select(4, self.sprite:rect())) / self.scene.pixelsPerUnit)
    end
    return vec2(1)
end

function entity:pixelSize()
    return vec2(select(3, self.sprite:rect()), select(4, self.sprite:rect()))
end

function entity:pixelSizeRelative()
    return self:pixelSize() * vec2(self.scale.x, self.scale.y)
end

function entity:setWidth(width, noAspect)
    local entityCurrentWidth = self:worldSize().x
    local newScale = width / entityCurrentWidth
    self.sx = self.sx * newScale
    if not noAspect then
        self.sy = self.sy * newScale
    end
end


function entity:setHeight(height, noAspect)
    local entityCurrentHeight = self:worldSize().y
    local newScale = height / entityCurrentHeight
    self.sy = self.sy * newScale
    if not noAspect then
        self.sx = self.sx * newScale
    end
end

entity.addN = entity.add

local customComponents = {}

function entity:getId()
    return self.scene.name .. ":" .. self.id
end

function entity:add(compon, ...)
    local component = self:addN(compon, ...)
    local entId = self:getId()
    
    --print(component)
    if compon ~= physics2d.body then
        if not customComponents[entId] then
            customComponents[entId] = {}
        end
        table.insert(customComponents[entId], component)
    end
    return component
end


local startTime = {}

function entity:setUpPhysics()
    function self:collisionBegan2d(colli)
        if self.scene.hasStarted and time.elapsed - startTime[self.scene.name] > 0.2 then
            for i, com in ipairs(self:customComponents()) do
                if com.onCollisionBegan ~= nil then
                    com:onCollisionBegan(colli)
                end
            end
        end
    end
    
    function self:collisionEnded2d(colli)
        if self.scene.hasStarted and time.elapsed - startTime[self.scene.name] > 0.2 then
            for i, com in ipairs(self:customComponents()) do
                if com.onCollisionEnded ~= nil then
                    com:onCollisionEnded(colli)
                end
            end
        end
    end
    
end

function entity:customComponents()
    local entId = self:getId()
    if customComponents[entId] == nil then
        customComponents[entId] = {}
    end
    return customComponents[entId]
end


local oTouched = scene.touched
function scene:touched(touch)
    if mouse then
        fixMouse(touch)
    end
    if self.hasStarted then
        oTouched(self, touch)
    end
end

scene.hasStarted = false


function scene:start()
    self.hasStarted = true
    startTime[self.name] = time.elapsed
    for k, ent in ipairs(self:getAllEntities()) do
        
        if ent.body2d then
            ent:setUpPhysics()
        end
        
        for i, com in ipairs(ent:customComponents()) do
            if com.start ~= nil then
                com:start()
            end
        end
    end
end


local originalUpdate = scene.update
function scene:update(ti)
    self.time.autoUpdate = false
    if self.hasStarted then
        originalUpdate(self, ti)
    end
end

function scene:getAllEntities()
    local allEntities = {}
    for k, enti in ipairs(self:entities()) do
        self:getEntity(allEntities, enti)
    end
    return allEntities
end

function scene:getAllChildren(allEntities, entit)
    for k, enti in ipairs(entit.children) do
        self:getEntity(allEntities, enti)
    end
end

function scene:getEntity(allEntities, enti)
    table.insert(allEntities, enti)
    if #enti.children > 0 then
        self:getAllChildren(allEntities, enti)
    end
end